/*
 * Copyright (C) 2008 Patrick Ohly
 */

#include "PrefixConfigNode.h"
#include "EvolutionSyncClient.h"

#include <boost/foreach.hpp>
#include <boost/algorithm/string/predicate.hpp>

PrefixConfigNode::PrefixConfigNode(const string prefix,
                                   const boost::shared_ptr<ConfigNode> &node) :
    m_prefix(prefix),
    m_node(node),
    m_readOnlyNode(node)
{
}

PrefixConfigNode::PrefixConfigNode(const string prefix,
                                   const boost::shared_ptr<const ConfigNode> &node) :
    m_prefix(prefix),
    m_readOnlyNode(node)
{
}

string PrefixConfigNode::readProperty(const string &property) const
{
    return m_readOnlyNode->readProperty(m_prefix + property);
}

void PrefixConfigNode::setProperty(const string &property,
                                 const string &value,
                                 const string &comment,
                                 const string *defValue)
{
    m_node->setProperty(m_prefix + property,
                        value,
                        comment,
                        defValue);
}

void PrefixConfigNode::readProperties(map<string, string> &props) const
{
    map<string, string> original;
    m_readOnlyNode->readProperties(original);

    BOOST_FOREACH(const StringPair &prop, original) {
        string key = prop.first;
        string value = prop.second;

        if (boost::starts_with(key, m_prefix)) {
            props[key.substr(m_prefix.size())] = value;
        }
    }
}

void PrefixConfigNode::removeProperty(const string &property)
{
    m_node->removeProperty(m_prefix + property);
}

void PrefixConfigNode::flush()
{
    if (!m_node.get()) {
        EvolutionSyncClient::throwError(getName() + ": read-only, flushing not allowed");
    }
    m_node->flush();
}
